#!/usr/bin/perl -w


sub usage {
	print STDERR <<"EOF";
usage: $0 demo-c-file-1 ...

Outputs to standard out a .c file which should compile
to the following C definitions:
	DemoDescription demo_descriptions[];
	int             num_demo_descriptions;

Used exclusively when compiling gdam-demo.
EOF
	exit 1;
}



usage unless scalar(@ARGV);




for $fname (@ARGV) {
    open F, "$fname" or die "couldn't open $fname: $!";
    while (defined($_=scalar(<F>))) {
	#print STDERR "got line $_";
	if (/^\/\* DESCRIPTION/) {
		#print STDERR "MATCHED*********************\n";
		my @description_pieces = ();
		my $function;
                while (defined($_=scalar(<F>))) {
			chomp;
			#print STDERR "subgot: $_\n";
			last if /^\s*\*\s*$/;
			s/^\s*\*\s*//;
			push @description_pieces, $_;
		}
		for (;;) {
			my $line = scalar(<F>);
			last if ($line =~ /\*\//);
			$line =~ s/^\s*\*\s*//;
			if ($line =~ s/^Function:\s*//) {
				$function = $line;
				next;
			}
			warn "unrecognizable line: $line";
		}
		if (!scalar(@description_pieces)) {
			die "no description:  invalid test";
		}
		if (!defined($function)) {
			die "no function:  invalid test";
		}
		#print STDERR "GOT FUNCTION $function\n";
		my $ent = {};
		$ent->{description} = \@description_pieces;
		$ent->{function} = $function;
		push(@entries, $ent);
	}
    }
}

			
print <<"EOF";
#include "../demodescription.h"

EOF

for (@entries) {
	my $function = $_->{function};
	print "void $function(GdamChannel* channel, guint mixer_id);\n";
}


print <<"EOF";

DemoDescription demo_descriptions[] = {
EOF

for (@entries) {
	my @description_pieces = @{$_->{description}};
	my $function = $_->{function};
	print "\t{\n";
	print "\t\t0,";
	for (@description_pieces) {
		print "\n\t\t\"$_ \"";
	}
	print ",\n";
	print "\t\t$function\n";
	print "\t},\n";
}
print "};\n\n";

print "int num_demo_descriptions = \n";
print "\t(sizeof(demo_descriptions) / sizeof(demo_descriptions[0]));\n";
