/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamrangefinder.h"
#include <ctype.h>

static void gdam_range_finder_init(GdamRangeFinder* range_finder)
{
	range_finder->device_number = -1;
}

static void got_number(GdamRangeFinder* range_finder)
{
	GdamMidiDevice* midi_device = GDAM_MIDI_DEVICE(range_finder);
	int cur_value = range_finder->cur_value;

	if (range_finder->device_number < 0) {
		range_finder->device_number = cur_value;
	} else {
		gdam_midi_device_emit_pitch_wheel(midi_device,
						  range_finder->device_number,
						  cur_value);
		range_finder->device_number = -1;
	}
}

static void parse_character(GdamRangeFinder* range_finder, int c)
{
	gboolean is_digit = isdigit(c);
	if (is_digit) {
		if (! range_finder->in_number) {
			range_finder->in_number = TRUE;
			range_finder->cur_value = 0;
		}
		range_finder->cur_value *= 10;
		range_finder->cur_value += (c - '0');
	} else {
		if (range_finder->in_number) {
			got_number(range_finder);
			range_finder->in_number = FALSE;
		}
	}
}

static gboolean range_finder_parser(GIOChannel*   io_channel,
                                    GIOCondition  condition,
	                            gpointer      user_data)
{
	GdamMidiDevice* device = GDAM_MIDI_DEVICE(user_data);
	GdamRangeFinder* range_finder = GDAM_RANGE_FINDER(user_data);
	unsigned char midi_buf[1024];
	int num_read;
	int index;
	GIOError io_error;
	io_error = g_io_channel_read(io_channel,
	                             midi_buf,
			             sizeof(midi_buf),
			             &num_read);
	if (io_error == G_IO_ERROR_AGAIN)
		return TRUE;
	if (io_error != G_IO_ERROR_NONE) {
		g_warning("error reading from midi device %s",
		          device->filename);
		return FALSE;
	}
	if (num_read <= 0) {
		g_warning("MIDI device closed...");
		return FALSE;
	}

	for (index = 0; index < num_read; index++) {
		int c = midi_buf[index];
		parse_character(range_finder, c);
	}
	return TRUE;
}


static void gdam_range_finder_class_init(GdamMidiDeviceClass* klass)
{
	klass->parse_raw = range_finder_parser;
}

GtkType gdam_range_finder_get_type()
{
	static GtkType range_finder_type = 0;
	if (!range_finder_type) {
		static const GtkTypeInfo range_finder_info = {
			"GdamRangeFinder",
			sizeof(GdamRangeFinder),
			sizeof(GdamRangeFinderClass),
			(GtkClassInitFunc) gdam_range_finder_class_init,
			(GtkObjectInitFunc) gdam_range_finder_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		GtkType parent = GDAM_TYPE_MIDI_DEVICE;
		range_finder_type = gtk_type_unique(parent,
		                                 &range_finder_info);
	}
	return range_finder_type;
}
